#include <msp/core/refptr.h>
#include <msp/io/file.h>
#include <msp/strings/format.h>
#include "mp3decoder.h"
#include "oggdecoder.h"
#include "sounddecoder.h"

using namespace std;

namespace Msp {
namespace AL {

unsupported_sound::unsupported_sound(const string &w):
	runtime_error(w)
{ }


SoundDecoder::SoundDecoder():
	source(0),
	freq(0),
	size(0),
	format(MONO8),
	eof_flag(false)
{ }

SoundDecoder::~SoundDecoder()
{
	delete source;
}

SoundDecoder *SoundDecoder::open_file(const string &fn)
{
	RefPtr<IO::BufferedFile> file = new IO::BufferedFile(fn);
	SoundDecoder *decoder = open_io(*file);
	decoder->source = file.release();
	return decoder;
}

SoundDecoder *SoundDecoder::open_io(IO::Seekable &io)
{
	char sig_buf[8];
	io.read(sig_buf, sizeof(sig_buf));
	string signature(sig_buf, sizeof(sig_buf));
	if(OggDecoder::detect(signature))
		return new OggDecoder(io);
	else if(Mp3Decoder::detect(signature))
		return new Mp3Decoder(io);
	else
	{
		string sig_hex;
		for(unsigned i=0; i<sizeof(sig_buf); ++i)
		{
			if(i)
				sig_hex += ' ';
			sig_hex += Msp::format("%02X", static_cast<unsigned char>(sig_buf[i]));
		}
		throw unsupported_sound(sig_hex);
	}
}

} // namespace AL
} // namespace Msp
